# -*- coding: utf-8 -*-
"""
Created on Thu May 11 14:04:09 2023

@author: kerb930
"""

import pandas as pd
import numpy as np
import random



#-----------------------------------------------------------------------------
# Slider Controller Definitions (slider number generated randomly for each house)
#-----------------------------------------------------------------------------
range_high_limit = 5
range_low_limit = -5
deadband = 0

#----------------------------------------------------------------------------
# Define price event. Price data for 72 hours. (Adding 24 to put a day of data ahead of simulation start)
#----------------------------------------------------------------------------
eventStart = 62 #62 = 38+24 #th hour of the simulation
eventDuration = 4
eventPrice = 300 

#-----------------------------------------------------------------------------
# Read in prices and cooling/heating schedules to manually generate controller
# output as a two player files (one for heating, one for cooling)
# Make sure times are formatted as 00:00:00 and values as numbers in excel file
# Remember to change clock on Controller_Substation1.glm to match 2-day event
#-----------------------------------------------------------------------------

columnData = ['Delivery Date','Hour Ending','Repeated Hour Flag', 'Settlement Point', 'Settlement Point Price']
data = pd.read_csv('Revised Prices_July.csv', names=columnData)
delDate = data['Delivery Date'].to_list()
hour = data['Hour Ending'].to_list()
flag = data['Repeated Hour Flag'].to_list()
sPoint = data['Settlement Point'].to_list()
sPrice = data['Settlement Point Price'].to_list()

# Initialize price data since Ercot Prices.csv defines time in hour-ending
price = [] #[47.03]
date = [] #['2022-07-17']
time = [] #['0:00:00']
for k in range(len(sPrice)):
    if ('HB_BUSAVG') in sPoint[k]:
        price.append(sPrice[k])
        date.append(delDate[k])
        time.append(hour[k])

    # Player files can't have commas between the date and time
    datetime = {'date': date, 'time': time,}
    df = pd.DataFrame(datetime)
    df.to_csv('date.csv', index=False, header=False, sep = ' ')

    columnDate = ['date']
    readdatetime = pd.read_csv('date.csv',header=None, names=columnDate)
    datetime = readdatetime['date']

# Incorporate event price into price data
idx = 0
outprice = []
while idx < eventStart:
    outprice.append(price[idx])
    idx += 1
idx = eventStart
while idx < eventStart + eventDuration:
    outprice.append(eventPrice)
    idx += 1
idx = eventStart + eventDuration
while idx < len(price):
    outprice.append(price[idx])
    idx += 1

# Output price data to file for plotting    
dfprice = pd.DataFrame({'outprice': outprice})
dfprice.to_csv('eventprice.csv', header='outprice')

std = []
avg = []

for idx in range(len(price)-24): #range(49):
    std.append(np.std(np.float_(price[idx:idx+24])))
    avg.append(np.average(np.float_(price[idx:idx+24])))
stdprice = pd.DataFrame({'rolling standard deviation': std, 'rolling average price': avg})
stdprice.to_csv('standardDeviation.csv', header={'rolling standard deviation','rolling average price'})

#-----------------------------------------------------------------------------
# Cooling Controller
#-----------------------------------------------------------------------------
random.seed(1) #fix slider settings between runs

cData = pd.read_csv('coolingedit.csv', low_memory=False)
for k in range(1096):
    x = random.uniform(.2,1)
    #x = 0.2 #Define slider setting for all houses if not randomized between .2 and 1
    range_low = range_low_limit*x
    range_high = range_high_limit*x
    ramp_high = 2 + (1-x) if x != 0 else 0
    ramp_low = ramp_high
    cooling_setpoint = []
    cooling_setpoint.append(cData.iloc[8:,k+1].to_list())
    cooling_setpoint = cooling_setpoint[0]
    idx = 0
    coolSet = []
# ALWAYS price responsive, definitions adapted from: https://github.com/gridlab-d/gridlab-d/blob/master/market/controller.cpp   
    for idx in range(len(price)-25): 
        if float(outprice[idx+24]) >= np.average(np.float_(price[idx:idx+24])) and (float(cooling_setpoint[idx+24]) + ((float(outprice[idx+24])-np.average(np.float_(price[idx:idx+24]))) * (abs(range_high)/(ramp_high*np.std(np.float_(price[idx:idx+24])))) + deadband)) < (float(range_high) + float(cooling_setpoint[idx+24])):
            coolSet.append(float(cooling_setpoint[idx+24]) + (float(outprice[idx+24])-np.average(np.float_(price[idx:idx+24]))) * (abs(range_high)/(abs(ramp_high)*np.std(np.float_(price[idx:idx+24])))) + deadband)
        elif float(outprice[idx+24]) > np.average(np.float_(price[idx:idx+24])):
            coolSet.append(float(cooling_setpoint[idx+24]) + float(range_high)) 
        elif float(outprice[idx+24]) <= np.average(np.float_(price[idx:idx+24])) and (float(cooling_setpoint[idx+24]) + ((float(outprice[idx+24])-np.average(np.float_(price[idx:idx+24]))) * (abs(range_low)/(ramp_low*np.std(np.float_(price[idx:idx+24])))) + deadband)) > (float(range_low) + float(cooling_setpoint[idx+24])):
            coolSet.append(float(cooling_setpoint[idx+24]) + (float(outprice[idx+24])-np.average(np.float_(price[idx:idx+24]))) * (abs(range_low)/(abs(ramp_low)*np.std(np.float_(price[idx:idx+24])))) + deadband)
        elif float(outprice[idx+24]) < np.average(np.float_(price[idx:idx+24])):
            coolSet.append(float(cooling_setpoint[idx+24]) + float(range_low))


# NO CONTROLLER: as scheduled HVAC usage
    # for idx in range(len(price)-25): #range(49):
    #     coolSet.append(float(cooling_setpoint[idx+24]))
            
    controllerC = {'date': datetime[25:], 'cooling_setpoint': coolSet}
    df2 = pd.DataFrame(controllerC)
    df2.to_csv(f'coolController_{cData.iloc[7,k+1]}.player', index=False, header=False)


#-----------------------------------------------------------------------------
# Heating Controller
#-----------------------------------------------------------------------------            

# hData = pd.read_csv('heating.csv')
# for k in range(1096):
#     x = np.random.random()
#     range_high = range_high_limit*x
#     ramp_high = -2-(1-x) if x != 0 else 0
#     ramp_high = ramp_low
#     heating_setpoint = []
#     heating_setpoint.append(hData.iloc[8:,k+1].to_list())     
#     heating_setpoint = heating_setpoint[0]
#     idx = 0
#     heatSet = []  
# ALWAYS price responsive, definitions adapted from: https://github.com/gridlab-d/gridlab-d/blob/master/market/controller.cpp   
    # for idx in range(len(outprice)):
    #     if float(outprice[idx+24]) >= np.average(np.float_(price[idx:idx+24])) and (float(heating_setpoint[idx+24]) + ((float(outprice[idx+24])-np.average(np.float_(price[idx:idx+24]))) * (abs(range_high)/(ramp_high*np.std(np.float_(price[idx:idx+24])))) + deadband)) < (float(range_high_limit) + float(heating_setpoint[idx+24])):
    #         heatSet.append(float(heating_setpoint[idx+24]) + ((float(outprice[idx+24])-np.average(np.float_(price[idx:idx+24]))) * (abs(range_low)/(ramp_low*np.std(np.float_(price[idx:idx+24])))) + deadband))
    #     elif float(outprice[idx+24]) > np.average(np.float_(price[idx:idx+24])):
    #         heatSet.append(float(heating_setpoint[idx+24]) + float(range_low_limit)) 
    #     elif float(outprice[idx+24]) <= np.average(np.float_(price[idx:idx+24])) and (float(heating_setpoint[idx+24]) + ((float(outprice[idx]+24)-np.average(np.float_(price[idx:idx+24]))) * (abs(range_low)/(ramp_low*np.std(np.float_(price[idx:idx+24])))) + deadband)) > (float(range_low_limit) + float(heating_setpoint[idx+24])):
    #         heatSet.append(float(heating_setpoint[idx+24]) + ((float(outprice[idx+24])-np.average(np.float_(price[idx:idx+24]))) * (abs(range_high)/(ramp_high*np.std(np.float_(price[idx:idx+24])))) + deadband))
    #     elif float(outprice[idx+24]) < np.average(np.float_(price[idx:idx+24])):
    #         heatSet.append(float(heating_setpoint[idx+24]) + float(range_high_limit))


# Brute force slider/override to prioritize cooling 
    # for idx in range(49):
    #     heatSet.append(float(heating_setpoint[idx+24]) - 15)
                
    # controllerH = {'date': datetime[24:122], 'heating_setpoint': heatSet}
    # df2 = pd.DataFrame(controllerH)
    # df2.to_csv(f'heatController_{hData.iloc[7,k+1]}.player', index=False, header=False)    