#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (C) 2019-2023 Battelle Memorial Institute
"""
Created on Tue Jan 17 14:05:08 2023

This script attempts to demonstrate the usage of the new TESP API to modify GridLAB-D models.
@author: hard312
@2ndauthor: kerb930
"""
import argparse
import logging
import pprint
import os
import sys

from tesp_support.api.modify_GLM import GLMModifier


# Setting up logging
logger = logging.getLogger(__name__)
pp = pprint.PrettyPrinter(indent=4)

# Adding custom logging level "DATA" to use for putting
#  all the simulation data on. "DATA" is between "DEBUG"
#  and "NOTSET" in terms of priority.
DATA_LEVEL_NUM = 5
logging.addLevelName(DATA_LEVEL_NUM, "DATA")
def data(self, message, *args, **kws):
    if self.isEnabledFor(DATA_LEVEL_NUM):
        self._log(DATA_LEVEL_NUM, message, args, **kws)
logging.DATA = DATA_LEVEL_NUM
logging.Logger.data = data

# Select feeder to build upon. Place in  ...\tesp-api-refactor\data\feeders
feeder_file = r'tesp-main\tesp-main\data\feeders\feeder_Controller_Substation1.glm'

def _auto_run(args):
    glmMod = GLMModifier()
    feeder_path = os.path.join(os.environ['TESPDIR'], feeder_file)
    glmMod.model.read(feeder_path)
    house_names = list(glmMod.glm.house.keys())
        
    #for k in range(len(house_names)):
    for k in range(250):

#-----------------------------------------------------------------------------
# Prep feeder file by retreiving and overwriting cooling/setpoint and adding
# billing meters
# Check climate data TX-Austin_Mueller_Municipal_Ap_Ut.tmy3
#-----------------------------------------------------------------------------        
        # glmMod.add_object_attr('house', house_names[k], 'cooling_setpoint', 70)
        glmMod.add_object_attr('house', house_names[k], 'heating_setpoint', 50)
        glmMod.add_object_attr('house', house_names[k], 'thermostat_deadband', 2)
        glmMod.add_object_attr('house', house_names[k], 'groupid', 'playerhouses')
        glmMod.add_object_attr('house', house_names[k], 'air_temperature', 75)
        glmMod.add_object_attr('house', house_names[k], 'mass_temperature', 75)

        # Adding billing meter to existing triplex meters 
        # This meter captures all energy usage for this customer
        # billing_meter_name = f'{tp_meter_names[k]}_billing'
        # billing_meter_params = {
        #     'parent': tp_meter_names[k],
        #     'phases': glmMod.get_object('triplex_meter').instance[tp_meter_names[k]]['phases'],
        #     'nominal_voltage': glmMod.get_object('triplex_meter').instance[tp_meter_names[k]]['nominal_voltage'],
        # }
        # meter_obj = glmMod.add_object('triplex_meter', billing_meter_name, billing_meter_params)

#------------------------------------------------------------------------------
# Add player objects to each house to define cooling/heating setpoint from
# controller.py generated player files (static market only)
# glm_objects.json needs a line added defining 'parent'
#------------------------------------------------------------------------------                

        cplayer_params =  {
                'parent': house_names[k],
                'property': 'cooling_setpoint',
                'file': f'coolController_{house_names[k]}.player'
                }
        load_obj = glmMod.add_object('player', f'coolController_{house_names[k]}.player', cplayer_params)
                    
        # hplayer_params =  {
        #     'parent': house_names[k],
        #     'property': 'heating_setpoint',
        #     'file': f'heatController_{house_names[k]}.player'
        #     }
        
        # load_obj = glmMod.add_object('player', f'heatController_{house_names[k]}.player', hplayer_params)       
    
#-----------------------------------------------------------------------------
# Add Recorders -- Note collectors have been added manually to the feeder file
#-----------------------------------------------------------------------------
        
            # air_recorder_params = {
            #     'group': '\"class=house\"',
            #     'property': 'air_temperature',
            #     'file': 'airtemp.csv',
            #     'interval': 60,
            #     }
            # recorder_obj = glmMod.add_object('group_recorder', 'air_temperature', air_recorder_params)
            
            # CoolON_recorder_params = {
            #     'group': '\"class=house\"',
            #     'property': 'is_COOL_on',
            #     'file': 'cool.csv',
            #     'interval': 60,
            #     }
            # recorder_obj = glmMod.add_object('group_recorder', 'Cool_On', CoolON_recorder_params)
            
            # HeatON_recorder_params = {
            #     'group': '\"class=house\"',
            #     'property': 'is_HEAT_on',
            #     'file': 'heat.csv',
            #     'interval': 60,
            #     }
            # recorder_obj = glmMod.add_object('group_recorder', 'HEAT_On', HeatON_recorder_params)
            
                      
        cooling_recorder_params = {
            'group': '\"class=house\"',
            'property': 'cooling_setpoint',
            'file': 'cooling.csv',
            'interval': 60
            }
        recorder_obj = glmMod.add_object('group_recorder', 'cooling_setpoint', cooling_recorder_params)
        
        heating_recorder_params = {
            'group': '\"class=house\"',
            'property': 'heating_setpoint',
            'file': 'heating.csv',
            'interval': 60
            }
        recorder_obj = glmMod.add_object('group_recorder', 'heating_setpoint', heating_recorder_params)
            
            # meterE_recorder_params = {
            #     'group': '\"class=meter\"',
            #     'property': 'measured_real_energy',
            #     'file': 'measured_real_energy.csv',
            #     'interval': 60
            #     }
            # recorder_obj = glmMod.add_object('group_recorder', 'measured_real_energy', meterE_recorder_params)
            
            # meterP_recorder_params = {
            #     'group': '\"class=meter\"',
            #     'property': 'measured_real_power',
            #     'file': 'measured_real_power.csv',
            #     'interval': 60
            #     }
            # recorder_obj = glmMod.add_object('group_recorder', 'measured_real_power', meterP_recorder_params)
            
            # substationE_recorder_params = {
            #     'group': '\"class=meter\"',
            #     'property': 'measured_real_power',
            #     'file': 'Substation_measured_real_power.csv',
            #     'interval': 60
            #     }
            # recorder_obj = glmMod.add_object('group_recorder', 'Substation_measured_real_power', substationE_recorder_params)
            
            # substationP_recorder_params = {
            #     'group': '\"class=meter\"',
            #     'property': 'measured_real_energy',
            #     'file': 'Substation_measured_real_energy.csv',
            #     'interval': 60
            #     }
            # recorder_obj = glmMod.add_object('group_recorder', 'Substation_measured_real_energy', substationP_recorder_params)
        
    glmMod.write_model(args.output_file)
        
        


if __name__ == '__main__':
    # TDH: This slightly complex mess allows lower importance messages
    # to be sent to the log file and ERROR messages to additionally
    # be sent to the console as well. Thus, when bad things happen
    # the user will get an error message in both places which,
    # hopefully, will aid in trouble-shooting.
    fileHandle = logging.FileHandler("feeder_generator.log", mode='w')
    fileHandle.setLevel(logging.DEBUG)
    streamHandle = logging.StreamHandler(sys.stdout)
    streamHandle.setLevel(logging.ERROR)
    logging.basicConfig(level=logging.DEBUG,
                        handlers=[fileHandle, streamHandle])

    parser = argparse.ArgumentParser(description='GridLAB-D Feeder Generator')
    script_path = os.path.dirname(os.path.realpath(__file__))
    parser.add_argument('-o',
                        '--output_file',
                        nargs='?',
                        default='Controller_Substation1.glm')
    args = parser.parse_args()
    _auto_run(args)